/* External functions

   OperStats - Copyright (c) 2000-2001 Darcy Grexton
   Contact: cygnus@uniserve.ca, skold @ HabberNet, ICQ #6285178

   This program is free but copyrighted software; see the file COPYING
   for details.
 */

#define E extern	/* Save us some typing */

/* socket.c */
E int sgetc (int s);
E char *sgets (char *buf, int len, int s);
E char *sgets2 (char *buf, int len, int s);
E int sputs (char *str, int s);
E int sockprintf (int s, char *fmt,...);
E int conn (const char *host, int port, const char *lhost, int lport);
E void disconn (int s);
E int32 total_read, total_written, speedsample;

/* main.c */
E int servsock;
E char *quitmsg;
E char *uplink;
E int servcnt, usercnt, chancnt;
E int servstat, userstat, chanstat, speedstat;
E int dailyuserstat, dailychanstat, dailyservstat;
E int maxservcnt, maxusercnt, maxchancnt;
E int debuglevel;
E time_t maxservtime, maxusertime, maxchantime, last_sample;
E time_t dailyusertime, dailychantime, dailyservtime;
E char inbuf[BUFSIZE];
E long runflags;
E void intro_user (const char *nick, const char *user, const char *host, const char *real, const char *modes);
E void checkpings (void);
E int is_one_of_mine (const char *nick);
E int servcount (void);
E const char version[];
E const char build[];
E const char compile_time[];
E const char *infotext[];
E const char *chanhtml_header[];
E const char *chanhtml_prechanlist[];
E const char *chanhtml_chanlist[];
E const char *chanhtml_preusercount[];
E const char *chanhtml_pretopics[];
E const char *chanhtml_endoflist[];
E const char *chanhtml_footer[];
E const char *chanhtml_applet[];

/* function.c */
E void open_log ();
E void open_debug ();
E void log (const char *fmt,...);
E void debug (const char *fmt,...);
E void close_log ();
E void close_debug ();
E void fatal (const char *fmt,...);
E void send_cmd (const char *source, const char *fmt, ...);
E void vsend_cmd (const char *source, const char *fmt, va_list args);
E void notice (const char *source, const char *dest, const char *fmt, ...); 
E void noticeall (const char *fmt, ...);
E void globops (const char *whoami, const char *fmt, ...);
E void ctcpreply (const char *source, const char *dest, const char *fmt, ...);
E int is_sra (const char *nick);
E int is_oper (const char *nick);
E int stricmp (const char *s1, const char *s2);
E void *smalloc (long size);
E void *scalloc (long elsize, long els);
E void *srealloc (void *oldptr, long newsize);
E char *sstrdup (const char *s);
E char *stristr (char *s1, char *s2);
E int match_wild (const char *pattern, const char *str);
E int match_wild_nocase (const char *pattern, const char *str);   
E char *strupper (char *s);
E char *strlower (char *s);
E void strip (char *line);
E void notice_list (const char *source, const char *dest, const char **text);
E SRA *findsra (const char *nick);
E TLD *findtld (const char *param);
E int is_akilled (const char *mask);
E int dotime (const char *s);
E char *create_mask (User *u);
E void snoop (const char *source, const char *fmt,...);
E char *duration (time_t event);
E char * strscpy (char *d, const char *s, size_t len);
E char *time_ago (time_t event);
E char *get_time (time_t event, int fulldate);
E char *stripcodes (char *buffer);
#ifdef DUMPCHANS
E void dumpchans (void);
#endif

/* process.c */
E ServStat *statlist[1024];
E ServStat *findserv (const char *server);
E int split_buf (char *buf, char ***argv, int colon_special);
E void process (void);

/* conf.c */
E SRA *sralist;
E void load_conf (void);
E int check_conf (void);
E char s_OperServ[NICKLEN], s_StatServ[NICKLEN], s_GlobalNoticer[NICKLEN];
E char tlds[64], snoopchan[32], def_akill_reason[256];
E Boolean_T operserv_on, statserv_on, globalnoticer_on, globalonakill_on;
E Boolean_T stamplogons_on, logupdates_on;
E int update_timeout, ping_delay, nonwildreq, def_akill_time, unreal_numeric;
E int wait_restart;

/* server.c */
E Servers *servlist;
E User *finduser (const char *nick);
E User *userlist[1024];
E Channel *findchan (const char *chan);
E Channel *chanlist[1024];
E void chan_adduser (User *user, const char *chan);
E void chan_deluser (User *user, Channel *c);
E void do_nick (const char *source, int ac, char **av);
E void do_join (const char *source, int ac, char **av);
E void do_part (const char *source, int ac, char **av);
E void do_kick (const char *source, int ac, char **av);
E void do_umode (const char *source, int ac, char **av);
E void do_quit (const char *source, int ac, char **av);
E void do_kill (const char *source, int ac, char **av);
E void do_umode (const char *source, int ac, char **av);
E void do_cmode (const char *source, int ac, char **av);
E void do_topic (const char *source, int ac, char **av);
E int countchans (int secret);
E void kill_user (const char *source, const char *user, const char *reason);
E int countusermask (const char *mask);
E User *findusermask (const char *mask, int matchno);
E void delete_user (User * user);
#ifdef BAHAMUT
E void do_sjoin (const char *source, int ac, char **av);
#endif

/* shared.c */
E void do_auth (const char *whoami, const char *source);
E void do_deauth (const char *whoami, const char *source);
E void do_raw (const char *whoami, const char *source);
E void do_inject (const char *whoami, const char *source);
E void do_debug (const char *whoami, const char *source);
E void do_shutdown (const char *whoami, const char *source);
E void do_restart (const char *whoami, const char *source);
E void do_settings (const char *whoami, const char *source);
E void do_update (const char *whoami, const char *source);
E void do_rehash (const char *whoami, const char *source);

/* operserv.c */
E AutoKill *akilllist[1024];
E MSG *msglist;
E void operserv (const char *source, char *buf);
E int check_akill (const char *nick, const char *user, const char *host, const char *real);
E void load_os_dbase (void);
E void save_os_dbase (void);
E void expire_akills (void);

/* statserv.c */
E void statserv (const char *source, char *buf);
E void load_ss_dbase (void);
E void save_ss_dbase (void);
E void load_tlds (void);
E void reset_daily_stats (void);
E TLD *tldlist;
