/* Processing code.

   OperStats - Copyright (c) 2000-2001 Darcy Grexton
   Contact: cygnus@uniserve.ca, skold @ HabberNet, ICQ #6285178

   This program is free but copyrighted software; see the file COPYING
   for details.
 */

#include "../inc/operstats.h"

Servers *servlist = NULL;
ServStat *statlist[1024];
int serv_size = 0;

/* split_buf:  Split a buffer into arguments and store the arguments in an
               argument vector pointed to by argv (which will be malloc'd
               as necessary); return the argument count.  If colon_special
               is non-zero, then treat a parameter with a leading ':' as
               the last parameter of the line, per the IRC RFC. Destroys
               the buffer by side effect.
 */
int split_buf (char *buf, char ***argv, int colon_special)
{
    int argvsize = 8;
    int argc;
    char *s;

    *argv = smalloc (sizeof(char *) * argvsize);
    argc = 0;
    while (*buf)
    {
	if (argc == argvsize)
	{
	    argvsize += 8;
	    *argv = srealloc (*argv, sizeof(char *) * argvsize);
	}
	if (*buf == ':')
	{
	    (*argv)[argc++] = buf+1;
	    buf = "";
	}
	else
	{
	    s = strpbrk(buf, " ");
	    if (s)
	    {
		*s++ = 0;
		while (isspace(*s))
		    s++;
	    }
	    else
		s = buf + strlen(buf);
	    (*argv)[argc++] = buf;
	    buf = s;
	}
    }
    return argc;
}

/* process:  Main processing routine. */
void process ()
{
    char source[64];
    char cmd[64];
    char buf[512];
    char *s;
    int ac;
    char **av;
    Message *m;

    /* If debugging, log the buffer. */
    if (debuglevel == 1 || debuglevel == 3)
	debug ("--> %s", inbuf);

    /* First make a copy of the buffer so we have the original in case we
       crash - in that case, we want to know what we crashed on.
     */
    strscpy (buf, inbuf, sizeof (buf));

    /* Split the buffer into pieces. */
    if (*buf == ':')
    {
	s = strpbrk (buf, " ");
	if (!s)
	    return;
	*s = 0;
	while (isspace(*++s))
	    ;
	strscpy (source, buf+1, sizeof(source));
	memmove (buf, s, strlen (s)+1);
    }
    else
	*source = 0;
    if (!*buf)
	return;
    s = strpbrk (buf, " ");
    if (s)
    {
	*s = 0;
	while (isspace (*++s))
	    ;
    }
    else
	s = buf + strlen(buf);
    strscpy (cmd, buf, sizeof(cmd));
    ac = split_buf (s, &av, 1);

    /* Do something with the message. */
    m = find_message (cmd);
    if (m)
    {
	if (m->func)
	    m->func (source, ac, av);
    }

    /* Free argument list we created */
    free (av);
}

/* Send our MOTD */
static void m_motd (char *source, int ac, char **av)
{
    FILE *f;
    char buf[BUFSIZE];

    send_cmd (me.name, "375 %s :- %s Message of the Day -", source, me.name);
    send_cmd (me.name, "372 %s :- ", source);
    send_cmd (me.name,
	"372 %s :- OperStats v%s Copyright (c) 2000-2001 Darcy Grexton",
	source, version);
    send_cmd (me.name, "372 %s :- ", source);

    f = fopen (MOTDFILE, "r");

    if (f)
    {
	while (fgets (buf, sizeof (buf), f))
	{
	    buf[strlen (buf) - 1] = 0;
	    send_cmd (me.name, "372 %s :- %s", source, buf);
	}
	fclose (f);
    }

    send_cmd (me.name, "376 %s :End of /MOTD command.", source);
}

/* Reply to a TIME query */
static void m_time (char *source, int ac, char **av)
{
    time_t t;
    struct tm *tm;
    char buf[80];

    time (&t);
    tm = localtime (&t);
    strftime (buf, sizeof (buf), "%A %B %d %Y -- %H:%M %z", tm);

    send_cmd (me.name, "391 %s %s :%s", source, me.name, buf);
}

/* Send an ADMIN reply */
static void m_admin (char *source, int ac, char **av)
{
    send_cmd (me.name, "256 %s :Administrative info about %s", source,
	me.name);
    send_cmd (me.name, "257 %s :OperStats v%s", source, version);
    send_cmd (me.name, "258 %s :Copyright (c) 2000-2001 Darcy Grexton",
	source);
    send_cmd (me.name,
	"259 %s :For more information E-Mail cygnus@uniserve.ca.", source);
}

/* Reply to a ping */
static void m_ping (char *source, int ac, char **av)
{
    if (ac < 1)
	return;

    send_cmd (me.name, "PONG %s %s", ac > 1 ? av[1] : me.name, av[0]);
}

/* A server replied to a PING, update its ping time */
static void m_pong (char *source, int ac, char **av)
{
    int i;

    if (ac < 1)
	return;

    for (i = 0; i < servcnt && stricmp (servlist[i].server, source); i++);
	if (i < servcnt)
	{
	    servlist[i].lag = time (NULL) - servlist[i].lastping;
	    servlist[i].lastpong = time (NULL);
	}
}

/* Handle a KICK command */
static void m_kick (char *source, int ac, char **av)
{
    if (ac != 3)
	return; 

    do_kick (source, ac, av);
}

/* Handle a TOPIC command */
static void m_topic (char *source, int ac, char **av)
{
    if (ac != 4)
	return;

    do_topic (source, ac, av);
}

/* Handle a JOIN command */
static void m_join (char *source, int ac, char **av)
{
    if (ac != 1)
	return;

    do_join (source, ac, av);
}

#ifdef BAHAMUT
/* Handle an SJOIN command. */
static void m_sjoin (char *source, int ac, char **av)
{
    do_sjoin (source, ac, av);
}
#endif

/* Handle a PART command */
static void m_part (char *source, int ac, char **av)
{
    if (ac < 1 || ac > 2)   
	return;

    do_part (source, ac, av);
}

/* An AKILL was placed elsewhere, learn it. */
static void m_akill (char *source, int ac, char **av)
{
    if (operserv_on == TRUE)
    {
	AutoKill *akill, **list;
	char *mask;

	mask = smalloc (strlen (av[1]) + strlen (av[0]) + 1);
	sprintf (mask, "%s@%s", av[1], av[0]);

	akill = scalloc (sizeof (AutoKill), 1);
	akill->mask = sstrdup (mask);
	akill->reason = sstrdup (av[2]);
	akill->setter = sstrdup (source);
	akill->realname = 0;
	akill->set = time (NULL);
	akill->expires = def_akill_time;
	list = &akilllist[AHASH(akill->mask)];
	akill->next = *list;
	if (*list)
	    (*list)->prev = akill;
	*list = akill;

	if (globalonakill_on == TRUE)
	    globops (s_OperServ, OS_AKILL_LEARNED, source, mask, av[2]);

	log (OS_AKILL_LEARNED, source, mask, av[2]);

	free (mask);
    }
}

/* Handle a NICK command */
static void m_nick (char *source, int ac, char **av)
{
    do_nick (source, ac, av);
}

/* Handle a QUIT command */
static void m_quit (char *source, int ac, char **av)
{
    if (ac != 1)
	return;

    do_quit (source, ac, av);
}

/* Send our version reply */
void m_version (char *source, int ac, char **av)
{
    if (source)
	send_cmd (me.name, "351 %s operstats-%s. %s %s%s%s%s",
	    source, version, me.name,
	    (operserv_on == TRUE) ? "O" : "",
	    (statserv_on == TRUE) ? "S" : "",
	    (globalnoticer_on == TRUE) ? "G" : "",
	    debuglevel ? "d" : "");
}

/* A server replied to a version query, store its reply. */
void m_vreply (char *source, int ac, char **av)
{
    int i;
    ServStat *server;

    for (i = 0; i < 1024; i++)
    {       
	for (server = statlist[i]; server; server = server->next)
	{
	    if (!stricmp (server->name, source))
	    {
		server->version = sstrdup (av[1]);
		server->flags = sstrdup (av[3]);
	    }
	}
    }
}

/* Reply to an INFO command */
void m_info (char *source, int ac, char **av)
{
    int i;

    for (i = 0; infotext[i]; i++)
	send_cmd (me.name, "371 %s :%s", source, infotext[i]);
    send_cmd (me.name, "371 %s :operstats-%s", source, version);
    send_cmd (me.name,
	      "371 %s :Birth Date: %s, compile # %s", source, compile_time,
	      build);
    send_cmd (me.name, "371 %s :On-line since %s", source, ctime (&me.since));
    send_cmd (me.name, "374 %s :End of /INFO list.", source);
}

#ifdef UNREAL3

/* Reply to a NETINFO command. */
void m_netinfo (char *source, int ac, char **av)
{
    /* If the server says it's seen a higher max than we have, update
       our numbers. The time will be off, but ah well :)
     */
    if (atoi (av[0]) > maxusercnt)
    {
	maxusercnt = atoi (av[0]);
	maxusertime = time (NULL);
    }

    send_cmd (me.name, "NETINFO %d %lu 0 * * * * :%s", maxusercnt, time (NULL),
	av[7]);
}

#endif /* UNREAL3 */

/* Reply to a STATS query */
void m_stats (char *source, int ac, char **av)
{
    if (ac < 1)
	return;

    switch (*av[0])
    {
	case 'u':
	{
	    int uptime = time (NULL) - me.since;
	    int tmp = servcount ();

	    send_cmd (me.name, "242 %s :Server Up %d days, %d:%02d:%02d",
		source, uptime/86400, (uptime/3600) % 24, (uptime/60) % 60,
		uptime % 60);
	    send_cmd (me.name,
		"250 %s :Highest connection count: %d (%d clients)",
		source, tmp + 1, tmp);
	    send_cmd (me.name, "219 %s u :End of /STATS report.", source);
	    break;
	}
	default:
	    send_cmd (me.name, "219 %s * :End of /STATS report.", source);
    }
}

/* Reply to a WHOIS query */
void m_whois (char *source, int ac, char **av)
{   
    if (ac < 1)
	return;   

    if ((!stricmp (av[0], s_OperServ)) && operserv_on == TRUE)
    {
	send_cmd (me.name, "311 %s %s %s %s * :%s", source, s_OperServ,
	    os.user, os.host, os.real);
	send_cmd (me.name, "312 %s %s %s :%s", source, s_OperServ,
	    me.name, me.desc);
	if (strchr (os.mode, 'o'))
	    send_cmd (me.name, "313 %s %s :is an IRC Operator", source,
		s_OperServ);
	send_cmd (me.name, "317 %s %s 0 %d :seconds idle, signon time",
	    source, s_OperServ, me.since);
    }
    else if ((!stricmp (av[0], s_StatServ)) && statserv_on == TRUE)
    {
	send_cmd (me.name, "311 %s %s %s %s * :%s", source, s_StatServ,
	    ss.user, ss.host, ss.real);
	send_cmd (me.name, "312 %s %s %s :%s", source, s_StatServ,
	    me.name, me.desc);
	if (strchr (ss.mode, 'o'))
	    send_cmd (me.name, "313 %s %s :is an IRC Operator", source,
		s_StatServ);
	send_cmd (me.name, "317 %s %s 0 %d :seconds idle, signon time",
	    source, s_StatServ, me.since);
    }
    else if ((!stricmp (av[0], s_GlobalNoticer)) && globalnoticer_on == TRUE)
    {
	send_cmd (me.name, "311 %s %s %s %s * :%s", source, s_GlobalNoticer,
	    gn.user, gn.host, gn.real);
	send_cmd (me.name, "312 %s %s %s :%s", source, s_GlobalNoticer,
	    me.name, me.desc);
	if (strchr (gn.mode, 'o'))
	    send_cmd (me.name, "313 %s %s :is an IRC Operator", source,
		s_GlobalNoticer);
	send_cmd (me.name, "317 %s %s 0 %d :seconds idle, signon time",
	    source, s_GlobalNoticer, me.since);
    }
    else
	send_cmd (me.name, "401 %s %s :No such nick/channel", source, av[0]);
    send_cmd (me.name, "318 %s %s :End of /WHOIS list", source, av[0]);
}

/* Handle a SERVER command */
static void m_server (char *source, int ac, char **av)
{
    ServStat *serverstat, **list;

    if (servcnt >= serv_size)
    {
	if (serv_size < 8)
	    serv_size = 8;
	else
	    serv_size *= 2;
	servlist = srealloc (servlist, sizeof (*servlist) * serv_size);
    }
    servlist[servcnt].server = sstrdup (av[0]);
    servlist[servcnt].uplink = sstrdup (source);
    servlist[servcnt].hops = atoi (av[1]);
    servlist[servcnt].lag = 999;
    servlist[servcnt].connect = time (NULL);
    servlist[servcnt].lastping = time (NULL);

    /* We'll only ping if StatServ is on, because the results aren't
       viewable otherwise. The VERSION reply isn't viewable either, but
       StatServ may be enabled later on.
     */
    send_cmd (me.name, "VERSION %s", av[0]);
    if (statserv_on == TRUE)
	send_cmd (me.name, "PING %s %s", me.name, av[0]);
    servcnt++;
    servstat++;
    if (servstat > maxservcnt)
    {
	maxservcnt = servstat;
	maxservtime = time (NULL);
    }
    if (maxservcnt > dailyservstat)
    {
	dailyservstat = servstat;
	dailyservtime = maxservtime;
    }

    /* We keep this in a seperate list because it's easier ;) */
    serverstat = findserv (av[0]); /* It may already be on record */

    if (!serverstat) /* Nope, add one for it. */
    {
	serverstat = scalloc (sizeof (ServStat), 1);

	serverstat->name = sstrdup (av[0]);
	serverstat->version = "Unknown";
	serverstat->flags = "XX";
	serverstat->statssince = time (NULL);
	serverstat->lastsplit = time (NULL);
	serverstat->is_split = 0;
	serverstat->splittime = 0;
	serverstat->splits = 0;
	list = &statlist[SHASH(serverstat->name)];
	serverstat->next = *list;
	if (*list)
	    (*list)->prev = serverstat;
	*list = serverstat;
    }
    else /* We already have it on record, and it just rejoined the network. */
	if (serverstat->is_split) /* Did it split, or was it lost in a split? */
	{
	    serverstat->splittime += time (NULL) - serverstat->lastsplit;
	    serverstat->is_split = 0;
	}
}

/* Handle an SQUIT command */
static void m_squit (char *source, int ac, char **av)
{
    int i, j = 0;
    User *u;
    ServStat *serverstat;

    for (i = 0; i < servcnt && stricmp (servlist[i].server, av[0]); i++);

    if (i < servcnt)
    {
	/* We give servers the NOQUIT token on link .. If they don't
	   understand it, they'll send QUITs first, so we won't find
	   any users for that server.
	 */
	while (j < 1023)
	{
	    j++;
	    u = userlist[j];
	    if (u && !stricmp (servlist[i].server, u->server))
	    {
		delete_user (u);
		u = u->next;
	    }
	}

	if (servlist[i].server)
	    free (servlist[i].server);
	if (servlist[i].uplink)
	    free (servlist[i].uplink);
	--servcnt;
	--servstat;
	if (i < servcnt)
	    bcopy (servlist + i + 1, servlist + i, sizeof
		(*servlist) * (servcnt - i));
    }

    /* We maintain a seperate list for stats */
    serverstat = findserv (av[0]);

    if (serverstat)
    {
	serverstat->lastsplit = time (NULL);
	serverstat->splits += 1;
	serverstat->is_split = 1;
    }
}

/* Handle a KILL command. */
static void m_kill (char *source, int ac, char **av)
{
    if (ac != 2)
	return;

    /* If one of ours was killed, respawn it */
    if ((operserv_on == TRUE) && !stricmp (av[0], s_OperServ))
	intro_user (s_OperServ, os.user, os.host, os.real, os.mode);
    else if ((statserv_on == TRUE) && !stricmp (av[0], s_StatServ))
	intro_user (s_StatServ, ss.user, ss.host, ss.real, ss.mode);
    else if ((globalnoticer_on == TRUE) && !stricmp (av[0], s_GlobalNoticer))
	intro_user (s_GlobalNoticer, gn.user, gn.host, gn.real, gn.mode);
    else
	do_kill (source, ac, av);
}

/* Handle a MODE command */
static void m_mode (char *source, int ac, char **av)
{
    if (*av[0] == '#')
    {
	if (ac < 2)
	    return;
	do_cmode (source, ac, av);
    }
    else
    {
	if (ac != 2)
	    return;
	do_umode (source, ac, av);
    }
}

/* Handle PRIVMSGs. */
static void m_privmsg (char *source, int ac, char **av)
{
    char buf[BUFSIZE];

    if (ac != 2)
	return;

    snprintf (buf, sizeof (buf), "%s@%s", s_OperServ, me.name);
    if (!stricmp (av[0], s_OperServ) || !stricmp (av[0], buf))
	operserv (source, av[1]);

    snprintf (buf, sizeof (buf), "%s@%s", s_StatServ, me.name);
    if (!stricmp (av[0], s_StatServ) || !stricmp (av[0], buf))
	statserv (source, av[1]);
}

Message messages[] = {
    {"KICK",	m_kick},
    {"KILL",	m_kill},
    {"MODE",	m_mode},
    {"NICK",	m_nick},
    {"PART",	m_part},
    {"PING",	m_ping},
    {"PONG",	m_pong},
    {"PRIVMSG",	m_privmsg},
    {"JOIN",	m_join},
    {"QUIT",	m_quit},
    {"SERVER",	m_server},
    {"SQUIT",	m_squit},
    {"VERSION",	m_version},
    {"INFO",	m_info},
    {"WHOIS",	m_whois},
    {"STATS",	m_stats},
    {"MOTD",	m_motd},
    {"TIME",	m_time},
    {"351",	m_vreply},
    {"ADMIN",	m_admin},
    {"AKILL",	m_akill},
    {"TOPIC",	m_topic},

    /* IRCd specific commands */
#ifdef UNREAL3
    {"NETINFO",	m_netinfo},
#endif

#ifdef BAHAMUT
    {"SJOIN",	m_sjoin},
#endif

    /* Tokens - DreamForge - DreamForge has an array of tokens defined,
       but only ever sends 5 tokenized commands, one of which we don't
       care about.
     */
    {"G",	m_mode},
    {"&",	m_nick},
    {"C",	m_join},
    {")",	m_topic},

    /* Tokens - UnrealIRCd. - Unreal sends more than just these, but 
       these are the only ones we care about right now. Others will be
       added later as we add new features. The 3 above are the same on
       Unreal.
     */
#ifdef UNREAL3
    {"H",       m_kick},
    {".",	m_kill},
    {"+",	m_version},
    {"/",	m_info},
    {"-",	m_squit},
    {"#",	m_whois},
    {"2",	m_stats},
    {">",	m_time},
    {"@",	m_admin},
    {"F",	m_motd},
    {"D",	m_part},
    {"8",	m_ping},
    {"9",	m_pong},
    {",",	m_quit},
    {"'",	m_server},
    {"!",	m_privmsg},
    {"AO",	m_netinfo},
#endif /* UNREAL3 */    
    {NULL}
};

Message *find_message (const char *name)
{
    Message *m;

    for (m = messages; m->name; m++)
	if (!stricmp (name, m->name))
	    return m;
    return NULL;
}
